#include <Adafruit_Fingerprint.h>

int lockPIN = 13;
int red_light_pin = 11;
int green_light_pin = 10;
int blue_light_pin = 9;
int buttonPIN = 4;
int buttonState = 0;
int buzzerPin = 12;

SoftwareSerial mySerial(2, 3);

Adafruit_Fingerprint finger = Adafruit_Fingerprint(&mySerial);

void RGB_color(int red_light_value, int green_light_value, int blue_light_value)
{
  analogWrite(red_light_pin, red_light_value);
  analogWrite(green_light_pin, green_light_value);
  analogWrite(blue_light_pin, blue_light_value);
}

void setup()
{
  // setup for lock
  pinMode(lockPIN, OUTPUT);
  digitalWrite(lockPIN, LOW);

  // led
  pinMode(red_light_pin, OUTPUT);
  pinMode(green_light_pin, OUTPUT);
  pinMode(blue_light_pin, OUTPUT);
  RGB_color(255, 0, 0);

  // button
  pinMode(buttonPIN, INPUT);

  // buzzer
  pinMode(buzzerPin, OUTPUT);

  Serial.begin(9600);
  while (!Serial);
  
  delay(100);
  Serial.println("\n\nAdafruit finger detect test");

  // set the data rate for the sensor serial port
  finger.begin(57600);
  delay(5);
  if (finger.verifyPassword()) {
    Serial.println("Found fingerprint sensor!");
  } else {
    Serial.println("Did not find fingerprint sensor :(");
    while (1) {
      delay(1);
    }
  }

  Serial.println(F("Reading sensor parameters"));
  finger.getParameters();
  Serial.print(F("Status: 0x")); Serial.println(finger.status_reg, HEX);
  Serial.print(F("Sys ID: 0x")); Serial.println(finger.system_id, HEX);
  Serial.print(F("Capacity: ")); Serial.println(finger.capacity);
  Serial.print(F("Security level: ")); Serial.println(finger.security_level);
  Serial.print(F("Device address: ")); Serial.println(finger.device_addr, HEX);
  Serial.print(F("Packet len: ")); Serial.println(finger.packet_len);
  Serial.print(F("Baud rate: ")); Serial.println(finger.baud_rate);

  finger.getTemplateCount();

  if (finger.templateCount == 0) {
    Serial.print("Sensor doesn't contain any fingerprint data. Please run the 'enroll' example.");
  }
  else {
    Serial.println("Waiting for valid finger...");
    Serial.print("Sensor contains "); Serial.print(finger.templateCount); Serial.println(" templates");
  }
}



uint8_t getFingerprintID() {
  uint8_t p = finger.getImage();
  switch (p) {
    case FINGERPRINT_OK:
      Serial.println("Image taken");
      break;
    case FINGERPRINT_NOFINGER:
      //Serial.println("No finger detected");
      return p;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("Communication error");
      return p;
    case FINGERPRINT_IMAGEFAIL:
      Serial.println("Imaging error");
      return p;
    default:
      Serial.println("Unknown error");
      return p;
  }

  // OK success!

  p = finger.image2Tz();
  switch (p) {
    case FINGERPRINT_OK:
      Serial.println("Image converted");
      break;
    case FINGERPRINT_IMAGEMESS:
      Serial.println("Image too messy");
      return p;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("Communication error");
      return p;
    case FINGERPRINT_FEATUREFAIL:
      Serial.println("Could not find fingerprint features");
      return p;
    case FINGERPRINT_INVALIDIMAGE:
      Serial.println("Could not find fingerprint features");
      return p;
    default:
      Serial.println("Unknown error");
      return p;
  }

  // OK converted!
  p = finger.fingerSearch();
  if (p == FINGERPRINT_OK) {
    Serial.println("Found a print match!");
  } else if (p == FINGERPRINT_PACKETRECIEVEERR) {
    Serial.println("Communication error");
    return p;
  } else if (p == FINGERPRINT_NOTFOUND) {
    Serial.println("Did not find a match");
    error();
    return p;
  } else {
    Serial.println("Unknown error");
    return p;
  }
  Serial.print("Found ID #"); Serial.print(finger.fingerID);
  Serial.print(" with confidence of "); Serial.println(finger.confidence);

  // found a match!
  unlock_door();
  return finger.fingerID;
}

void error()
{
  Serial.println("Fingerprint not matched. Alarm starts!");
  RGB_color(0, 0, 0);
  tone(buzzerPin, 315);
  delay(500);
  RGB_color(255, 0, 0);
  noTone(buzzerPin);
  delay(500);
  RGB_color(0, 0, 0);
  tone(buzzerPin, 315);
  delay(500);
  RGB_color(255, 0, 0);
  noTone(buzzerPin);
  delay(500);
}

void unlock_door()
{
  digitalWrite(lockPIN, HIGH);
  RGB_color(0, 255, 0); // Green
  Serial.println("Door unlocked for 2 seconds");
  tone(buzzerPin, 1000);
  delay(2000);
  noTone(buzzerPin);
  Serial.println("Door locked again");
  digitalWrite(lockPIN, LOW);
  RGB_color(255, 0, 0);
}



void loop()
{
  getFingerprintID();
  buttonState = digitalRead(buttonPIN);
  if (buttonState == HIGH) {
    unlock_door();
  }

  delay(200);
}
